import pickle
import pandas as pd
import re
import numpy as np

from urduistics.utils import load_resource

def preprocess(words):
    for i in range(0,len(words)):
        words[i] = re.sub(r'[^\w+]',' ',words[i])
    df = pd.DataFrame(list(zip(words)),columns=["words"])
    temp = df.copy()
    temp.replace(' ', np.nan, inplace=True)
    temp.dropna(inplace=True)
    return temp

def inverse_label(a):
    if a==0:
        return "Location"
    elif a==1:
        return "Organization"
    elif a==2:
        return "Other"
    elif a==3:
        return "Person"

def change_structure(sent):
    word_list = sent.split(' ')
    return pd.DataFrame(word_list)

def tfidfvectorizer(df):
    tfidf = load_resource("TFIDF_VECTORIZER","pkl")
    x = tfidf.transform(df['words'])
    return x

def model(data):
    pickle_model = load_resource("URDU_NER_MODEL","pkl")
    prediction = pickle_model.predict(data)
    return prediction

def ner_tagger(df):
    """
    This Function returns a list of words with their tags
    i.e Location, Organization, Person, Other.
    
        Arguments:
                df (DataFrame): DataFrame containing sentences in every row
        Returns: 
                word_list (List): List containing words and their tags
        Example:
            >>> df = pd.DataFrame(["میں امریکہ یونیسیف قاسم"])
            >>> word_list = ner_tagger(df)
            >>> print(word_list)
                [['میں', 'Other'],
                ['امریکہ', 'Location'],
                ['یونیسیف', 'Organization'],
                ['قاسم', 'Person']]
            >>> df = pd.DataFrame(["مضمون دارالعلوم شہابیہ کے جلیل"])
            >>> word_list = ner_tagger(df)
            >>> print(word_list)
                [['مضمون', 'Other'],
                ['دارالعلوم', 'Other'],
                ['شہابیہ', 'Organization'],
                ['کے', 'Other'],
                ['جلیل', 'Person']]
    """
    l = []
    for i in range(len(df)):
        w = (df[0][i].split(' '))
        for x in w:
            if x!='':
                l.append(x)
    word_list = preprocess(l)
    data = tfidfvectorizer(word_list)
    data = model(data)

    word_list['tags']=data
    word_list['tags'] = word_list['tags'].apply(lambda x: inverse_label(x))
    word_list = word_list.values.tolist()
    return word_list