import pandas as pd

from urduistics.utils import load_resource

def pos_tagger(sentence):
    mapping = {26: 'PN',
     12: 'G',
     21: 'NN',
     23: 'P',
     37: 'U',
     38: 'VB',
     35: 'SM',
     25: 'PM',
     27: 'PP',
     8: 'CC',
     3: 'ADJ',
     7: 'CA',
     32: 'RP',
     33: 'SC',
     34: 'SE',
     4: 'ADV',
     10: 'EXP',
     14: 'I',
     20: 'NEG',
     36: 'TA',
     6: 'AP',
     28: 'Q',
     24: 'PD',
     39: 'WALA',
     18: 'KP',
     13: 'GR',
     31: 'REP',
     0: 'A',
     16: 'KD',
     1: 'AA',
     29: 'QW',
     17: 'KER',
     22: 'OR',
     5: 'AKP',
     19: 'MUL',
     15: 'INT',
     2: 'AD',
     11: 'FR',
     9: 'DATE',
     30: 'RD'}
    punc = []
    punctuation = '!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~؟۔'
    for i in sentence:
        if i in punctuation:
            punc.append(i)
            sentence = sentence.replace(i, '')
    pickled_model = load_resource("URDU_POS_MODEL","pkl")
    s = sentence.split(' ')
    s = s + punc
    tags = []
    for i in range(len(s)):
        tag = pickled_model.predict([s[i]])
        tags.append((s[i], mapping[tag[0]]))
    return tags

def pos_tagger_dataframe(urdu):
    """
        This Function takes a dataframe or list consisting of sentences in Urdu language.
        The output will consist of a words from each sentence against their respective POS tags. The output will be returned as both, a list and a pandas DataFrame. Most punctuation is automatically dealt with.

        Arguments:
                    DataFrame or list with Urdu sentences (string)
        Returns:
                    DataFrame or list of words against their POS tags
        Example:
            >>> text = '-میرے بھائی کا ای میل آیاہے'
            Note that this sentence is a row in the DataFrame taken as input
            >>> Result = [('میرے', 'G'), ('بھائی', 'NN'), ('کا', 'P'), ('ای', 'PN'), ('میل', 'U'), ('آیاہے', 'VB'), ('-', 'NN')]
            Note that this result will also be stored in an output DataFrame row
    """
    u = urdu.copy()
    all_results = []
    all_df_results = []
    for i in u:
        all_results.append(pos_tagger(i))
        all_df_results.append(str(pos_tagger(i)))
        
    d = pd.DataFrame(all_df_results, columns=['TAGS'])
    return all_results, d