import spacy

from urduistics.utils import load_resource

unlp = spacy.blank('ur')

def word_tokenize(s):
    """
    This function tokenizes sentences to words in Urdu text. It not only separates single words, but compound words as well. These words are two or more different words intended for the same meaning or phrase. The text taken as input is preprocessed and the punctuation is not taken into consideration. The words taken as input are searched across an already built dictionary of Urdu compound words. This dictionary can be improved overtime with more and more data.

    Example: 
        >>> Input: 'یہ اسلامی جمہوریہ پاکستان کے موجودہ وزیر اعظم نواز شریف کی ذمہ داری ہے کہ وہ بانی پاکستان قائد اعظم کی ہدایات پر عمل کریں صدر مملکت پاکستان نے علامہ اقبال کو خراج عقیدت پیش کیا ہے'
        >>> word_tokenize(Input)
        >>> Output:['یہ', 'اسلامی جمہوریہ', 'پاکستان', 'کے', 'موجودہ', 'وزیر اعظم', 'نواز شریف', 'کی', 'ذمہ داری', 'ہے', 'کہ', 'وہ', 'بانی', 'پاکستان', 'قائد اعظم 'کی', 'ہدایات', 'پر', 'عمل', 'کریں', 'صدر مملکت', 'پاکستان', 'نے', 'علامہ اقبال', 'کو', 'خراج عقیدت', 'پیش', 'کیا', 'ہے']
        
    """
    word_tokens2 = []
    punc = '''!()%\n٪-;۔،:\n\/'"\,“./؟_ء'''
    cl = load_resource("URDU_WTK_MODEL","pkl")

    sen = unlp(s)
    for w in sen:
        #converting the word back to 'str'
        w = str(w)
        #preprocessing (removing punctuations)
        for c in w:
            if c in punc:
                w = w.replace(c,'')

        if w != "\n" and w != "\r" and w != " " and w != '' and w != " \r" and w !='‘' and w !='’':
            word_tokens2.append(w)

    for i in range(len(word_tokens2)):
        if i+1 < len(word_tokens2):
            # print(f"{word_tokens2[i]} , {word_tokens2[i+1]}")
            a = word_tokens2[i]
            b = word_tokens2[i+1]

            if a+' '+b in cl.keys():
                index = word_tokens2.index(a)
                word_tokens2.remove(a)
                word_tokens2.remove(b)
                word_tokens2.insert(index, a+' '+b)
    return word_tokens2