import os
import pickle
import pandas as pd
import json

from .io import download_url
from urduistics.conf import RESOURCES_DIR
from urduistics.conf import RESOURCES_URLS
from urduistics.conf import RESOURCES_PATHS

def download_resources():
    os.makedirs(RESOURCES_DIR, exist_ok=True)
    for res_name,url in RESOURCES_URLS.items():
        if not is_resource_available(res_name):
            download_url(url, RESOURCES_DIR)

def is_resource_available(res_name=""):
    if res_name not in [key for key,_ in RESOURCES_PATHS.items()] or not os.path.exists(RESOURCES_PATHS[res_name]):
        return False
    return True

def load_resource(res_name="", load_type="custom"):
    if not is_resource_available(res_name):
        output_msg = f"""RESOURCE: {res_name} was not found. 
Please type `urduistics download` in the terminal to download all resources"""
        print(output_msg)
        raise SystemExit
    elif load_type not in ['pkl','csv','json','custom']:
        print("Invalid load type.")
        raise SystemExit
    else:
        res = None
        if load_type == "pkl":
            with open(RESOURCES_PATHS[res_name], 'rb') as handle:
                res = pickle.load(handle)
        elif load_type == "csv":
            res = pd.read_csv(RESOURCES_PATHS[res_name])
        elif load_type == "json":
            with open(RESOURCES_PATHS[res_name], "r", encoding='utf-8') as f:
                res = json.loads(f.read())
        elif load_type == "custom":
            if res_name == "URDU_STEMMER_DATASET":
                res = {"PrGEL":None,"PoGEL":None,"Prefixes":None,"Postfixes":None,"CAL":None}
                for name,_ in res.items():
                    with open(os.path.join(RESOURCES_PATHS[res_name],name),'r',encoding='utf-8') as f:
                        res[name] = [word.strip() for word in f.read().split('\n')]
                        if name in ["PoGEL","PrGEL"]:
                            res[name] = frozenset(res[name])
        return res
